#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Menu systems for Storm Games.

Provides functionality for:
- Game menu navigation
- Instructions display
- Credits display
- Sound learning interface
- Game exit handling
"""

import pygame
import time
import webbrowser
import os
from sys import exit
from os.path import isfile
from os import listdir
from os.path import join
from inspect import isfunction
from .speech import messagebox, Speech
from .sound import adjust_master_volume, adjust_bgm_volume, adjust_sfx_volume, play_bgm
from .display import display_text
from .scoreboard import Scoreboard
from .services import PathService, ConfigService

def game_menu(sounds, playCallback=None, *customOptions):
    """Display and handle the main game menu with standard and custom options.
    
    Standard menu structure:
    1. Play (always first)
    2. High Scores (if scores exist)
    3. Custom options (if provided)
    4. Learn Sounds
    5. Instructions (if available)
    6. Credits (if available)
    7. Donate
    8. Exit
    
    Handles navigation with:
    - Up/Down arrows for selection
    - Home/End for first/last option
    - Enter to select
    - Escape to exit
    - Volume controls (with Alt modifier)
        
    Args:
        sounds (dict): Dictionary of sound objects
        playCallback (function, optional): Callback function for the "play" option.
            If None, "play" is returned as a string like other options.
        *customOptions: Additional custom options to include after play but before standard ones
        
    Returns:
        str: Selected menu option or "exit" if user pressed escape
    """
    # Get speech instance
    speech = Speech.get_instance()
    
    # Start with Play option
    allOptions = ["play"]
    
    # Add high scores option if scores exist
    if Scoreboard.hasHighScores():
        allOptions.append("high_scores")
    
    # Add custom options (other menu items, etc.)
    allOptions.extend(customOptions)
    
    # Add standard options in preferred order
    allOptions.append("learn_sounds")
    
    # Check for instructions file
    if os.path.isfile('files/instructions.txt'):
        allOptions.append("instructions")
        
    # Check for credits file
    if os.path.isfile('files/credits.txt'):
        allOptions.append("credits")
        
    # Final options
    allOptions.extend(["donate", "exit_game"])
    
    # Track if music was previously playing
    musicWasPlaying = pygame.mixer.music.get_busy()
    
    # Only start menu music if no music is currently playing
    if not musicWasPlaying:
        try:
            from .sound import play_bgm
            play_bgm("sounds/music_menu.ogg")
        except:
            pass
            
    loop = True
    pygame.mixer.stop()
    currentIndex = 0
    lastSpoken = -1  # Track last spoken index
    
    while loop:
        if currentIndex != lastSpoken:
            speech.speak(allOptions[currentIndex])
            lastSpoken = currentIndex
            
        event = pygame.event.wait()
        if event.type == pygame.KEYDOWN:
            # Check for Alt modifier
            mods = pygame.key.get_mods()
            altPressed = mods & pygame.KMOD_ALT
            
            # Volume controls (require Alt)
            if altPressed:
                if event.key == pygame.K_PAGEUP:
                    adjust_master_volume(0.1)
                elif event.key == pygame.K_PAGEDOWN:
                    adjust_master_volume(-0.1)
                elif event.key == pygame.K_HOME:
                    adjust_bgm_volume(0.1)
                elif event.key == pygame.K_END:
                    adjust_bgm_volume(-0.1)
                elif event.key == pygame.K_INSERT:
                    adjust_sfx_volume(0.1)
                elif event.key == pygame.K_DELETE:
                    adjust_sfx_volume(-0.1)
            # Regular menu navigation (no Alt required)
            else:
                if event.key == pygame.K_ESCAPE:
                    # Exit with fade if music is playing
                    exit_game(500 if pygame.mixer.music.get_busy() else 0)
                elif event.key == pygame.K_HOME:
                    if currentIndex != 0:
                        currentIndex = 0
                        try:
                            sounds['menu-move'].play()
                        except:
                            pass
                        if allOptions[currentIndex] != "donate":
                            pygame.mixer.music.unpause()
                elif event.key == pygame.K_END:
                    if currentIndex != len(allOptions) - 1:
                        currentIndex = len(allOptions) - 1
                        try:
                            sounds['menu-move'].play()
                        except:
                            pass
                        if allOptions[currentIndex] != "donate":
                            pygame.mixer.music.unpause()
                elif event.key in [pygame.K_DOWN, pygame.K_s] and currentIndex < len(allOptions) - 1:
                    currentIndex += 1
                    try:
                        sounds['menu-move'].play()
                    except:
                        pass
                    if allOptions[currentIndex] != "donate":
                        pygame.mixer.music.unpause()
                elif event.key in [pygame.K_UP, pygame.K_w] and currentIndex > 0:
                    currentIndex -= 1
                    try:
                        sounds['menu-move'].play()
                    except:
                        pass
                    if allOptions[currentIndex] != "donate":
                        pygame.mixer.music.unpause()
                elif event.key == pygame.K_RETURN:
                    try:
                        lastSpoken = -1
                        try:
                            sounds['menu-select'].play()
                            time.sleep(sounds['menu-select'].get_length())
                        except:
                            pass
                        
                        selectedOption = allOptions[currentIndex]
                        
                        # Special case for exit_game with fade
                        if selectedOption == "exit_game":
                            exit_game(500 if pygame.mixer.music.get_busy() else 0)
                        # Special case for play option
                        elif selectedOption == "play":
                            if playCallback:
                                # If a play callback is provided, call it directly
                                try:
                                    pygame.mixer.music.fadeout(500)
                                    time.sleep(0.5)
                                except:
                                    pass
                                playCallback()
                            else:
                                # Otherwise return "play" to the caller
                                return "play"
                        # Handle standard options directly
                        elif selectedOption in ["instructions", "credits", "learn_sounds", "high_scores", "donate"]:
                            # Pause music before calling the selected function
                            try:
                                pygame.mixer.music.pause()
                            except:
                                pass
                                
                            # Handle standard options
                            if selectedOption == "instructions":
                                instructions()
                            elif selectedOption == "credits":
                                credits()
                            elif selectedOption == "learn_sounds":
                                learn_sounds(sounds)
                            elif selectedOption == "high_scores":
                                Scoreboard.displayHighScores()
                            elif selectedOption == "donate":
                                donate()
                            
                            # Unpause music after function returns
                            try:
                                # Check if music is actually paused before trying to unpause
                                if not pygame.mixer.music.get_busy():
                                    pygame.mixer.music.unpause()
                                # If music is already playing, don't try to restart it
                            except:
                                # Only start fresh music if no music is playing at all
                                if not pygame.mixer.music.get_busy():
                                    try:
                                        from .sound import play_bgm
                                        play_bgm("sounds/music_menu.ogg")
                                    except:
                                        pass
                        # Return custom options to the calling function
                        else:
                            lastSpoken = -1
                            try:
                                pygame.mixer.music.fadeout(500)
                                time.sleep(0.5)
                            except:
                                pass
                            return selectedOption
                    except Exception as e:
                        print(f"Error handling menu selection: {e}")
                        lastSpoken = -1
                        try:
                            pygame.mixer.music.fadeout(500)
                            time.sleep(0.5)
                        except:
                            pass
                        return allOptions[currentIndex]
                    
        event = pygame.event.clear()
        time.sleep(0.001)

# Rest of menu.py functions here...
# (learn_sounds, instructions, credits, donate, exit_game)
